"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = combineReducers;

var _size2 = _interopRequireDefault(require("lodash/size"));

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _forEach2 = _interopRequireDefault(require("lodash/forEach"));

var _reduce2 = _interopRequireDefault(require("lodash/reduce"));

var _immutable = require("immutable");

var _mindfrontReduxUtils = require("mindfront-redux-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function combineReducers(reducers, createInitialState = _immutable.Map) {
  const actionHandlers = {};
  const otherReducers = {};
  const initialStateObj = {}; // regroup reducers that have actionHandlers:
  //   from: prop name -> action type -> reducer
  //     to: action type -> prop name -> reducer
  // put reducers without actionHandlers into a prop name -> reducer map

  (0, _forEach2.default)(reducers, (reducer, key) => {
    initialStateObj[key] = reducer(undefined, {});

    if (reducer.actionHandlers) {
      (0, _forEach2.default)(reducer.actionHandlers, (actionHandler, type) => {
        ;
        (actionHandlers[type] || (actionHandlers[type] = {}))[key] = actionHandler;
      });
    } else otherReducers[key] = reducer;
  });
  const initialState = createInitialState(initialStateObj); // now we can create a more efficient reducer for each action type
  // and one more efficient reducer for the otherReducers
  // creates an efficient reducer from a prop name -> reducer map

  const combineBase = reducers => {
    let result;

    if ((0, _size2.default)(reducers) > 1) {
      // we have to update multiple keys; use state.withMutations
      result = (state = initialState, action) => state.withMutations(mutableState => (0, _reduce2.default)(reducers, (nextState, reducer, key) => nextState.update(key, value => reducer(value, action)), mutableState));
    } else {
      // we only have to update one key; use state.update
      const key = Object.keys(reducers)[0];
      const reducer = reducers[key];

      result = (state = initialState, action) => state.update(key, value => reducer(value, action));
    }

    result.domainHandlers = reducers;
    return result;
  };

  const actionHandlerReducer = (0, _size2.default)(actionHandlers) ? (0, _mindfrontReduxUtils.createReducer)(initialState, (0, _mapValues2.default)(actionHandlers, combineBase)) : undefined;
  const otherReducer = (0, _size2.default)(otherReducers) ? combineBase(otherReducers) : undefined;
  if (actionHandlerReducer && otherReducer) return (0, _mindfrontReduxUtils.composeReducers)(actionHandlerReducer, otherReducer);
  if (actionHandlerReducer) return actionHandlerReducer;
  if (otherReducer) return otherReducer;
  return (state = initialState) => state;
}